
/**********************************************************************
* Module:	getopt.o
* Author:
*	-Kev	(mailto:kevin_bertram@hp.com)
*
*     Copyright (c) 2001 Hewlett Packard Pty. Ltd., All Rights Reserved.
*     
*     This program is free software; you can redistribute it and/or 
*     modify it under the terms of the GNU General Public License as 
*     published by the Free Software Foundation; either version 2 of 
*     the License, or (at your option) any later version.
* 
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*     GNU General Public License for more details.
* 
*     You should have received a copy of the GNU General Public License 
*     along with this program; if not, write to the Free Software 
*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
*     MA 02111-1307 USA
*
****************************************************************************
*
*  Description:
*	This module is used to search the command line arguments and
*	decompile them into a structure from which the program will be
*	driven from.
*
*	The controlling structure "gFlags" is first initialized to a default
*	behaviour the program should assume and then that behaviour will be
*	modified by each new command line switch found.
*
*  Note: The gFlags global structure is required by all modules and
*        controls the general actions of all modules and functions.
*
***********************************************************************
*  Revision History:
*  	Rev 1.00		22-September-1997	-Kev
*	(1)	First writting.
*
*	Rev 1.01		12-Feb-2001			-Kev
*	(1)	Adapted to IP probe daemon.
*
*	Rev 1.02		26-April-2001		-Kev
*	(1)	Added help messages controlled by the program's name.
*
*   Rev 1.03		6-July-2001			-Kev
*   (1)	Ported to suit irnetd irfred server daemon code.
*
*   Rev 1.04        9-August-2001       -Kev
*   (1) Added an option (-l) to prevent modifications being made to the
*       /etc/hosts file. -l for no logging of host names. Normally the
*       program will write out to the hosts file a name and IP address
*       for each new device PPP connects to. When this option is given
*       the hosts file remains untouched and the device is not registered
*       anyway.
*   (2) Added the softswitch (-f) to turn on/off daemon mode.
*
*   Rev 1.05        10-August-2001      -Kev
*   (1) Changed (-f) softswitch to (-D).
*
*	Rev 1.06		8-October-2001		-Kev
*	(1)	Added "-m <addrs>" option to hardware an IP address.
*	(2)	Made "-i" switch imply "-t".
*	(3)	Made "-t" switch imply "-i 15"
*	(4)	First big release.
***********************************************************************/

#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<time.h>
#include	<ctype.h>

/**** Read in locally defined includes for this module ****/

#include	"irnetd.h"
#include	"getopt.h"
#include	"files.h"


/**** Define the local functions needed only by this module ****/

	int	atox( char * );
	int	GetFatCommands( int argc, char ** argv );


/**** Define globals needed by the system ****/

	gFlagsType		gFlags;		/* Global program controls */



/***********************************************************************
 This function extracts command line arguments from the command line
 and installs them into the gFlags global structure.

 Currently supported softswitches list can be found in the ShowUsage
 function. This list is also used to display help information to the
 user when they get a command argument wrong.

 Returns:
 	Count of softswitches found.
 	-1	-	Bad switch encountered.
************************************************************************/

int	GetOpt( int argc, char **argv )
{
	int			i = 0;
	int			temp;
	int			arg_count = 0;
	char *		p;
	char		idle_string[ 80 ];

/**** Preload structure with defaults ****/

	gFlags.Mode         = MODE_HOSTS_FILE | MODE_DAEMON;
	gFlags.Idle         = 0;
	gFlags.Myaddress    = -1;
	gFlags.Show_usage   = FALSE;
	gFlags.Show_version = FALSE;
	gFlags.Debug_level  = 0;

	strcpy( gFlags.Pppcmd, PPP_CMD2 );
	memset( gFlags.Daddr, 0, sizeof(gFlags.Daddr) );

/**** Touch up the program name and present it for use by reporting code ****/

	p = argv[0];

	while (*p) p++;

	while ( (p != argv[0]) && (*p != '/')) p--;

	if (p != argv[0]) p++;

	strcpy( gFlags.Program, p );

/**** Recompose the arguments into one long string with space seperators ****/

	memset( gFlags.Command, 0, sizeof(gFlags.Command) );

	for (i=0; i < argc; i++)
	{
		if (argv[i])
		{
			strcat( gFlags.Command, argv[i] );
			strcat( gFlags.Command, " " );
		}
	}

/**** Try to find any fat keywords in command line ****/

	GetFatCommands( argc, argv );

/**** Begin searching for command line switches ****/

	p = gFlags.Command;

	while (*p)
	{
		while ( (*p == ' ') && (*p) ) p++;

		if ( (*p == '-') || (*p == '+') )
		{
			p++;
			arg_count++;

			switch (*p)
			{
				case ('h'):
				{
					gFlags.Show_usage = TRUE;
					break;
				}

/***** This section sets the level of the debug reporting ****/

				case ('d'):
				{
					p++;
					while ( (*p) && (*p == ' ') ) p++;

					if (*p != '-')
					{
						gFlags.Debug_level = atoi( p );
					}

					if (gFlags.Debug_level < 0)
					{
						gFlags.Debug_level = 0;
					}

					break;
				}

/***** Flag don't run as daemon mode ****/

				case ('D'):
				{
					gFlags.Mode &= ~MODE_DAEMON;
					break;
				}

/***** specify and idle timeout value for PPPD ****/

				case ('i'):
				{
					p++;
					while ( (*p) && (*p == ' ') ) p++;

					if (*p != '-')
					{
						gFlags.Idle = atoi( p );
					}

					if (gFlags.Idle < 0)
					{
						gFlags.Idle = 0;
					}

					gFlags.Mode |= MODE_TIMER_ONLY;
					break;
				}

/***** Turn off hosts file registering ****/

				case ('l'):
				{
					gFlags.Mode &= ~MODE_HOSTS_FILE;
					break;
				}

/**** Extract the user specified IP address ****/

				case ('m'):
				{
					p++;
					while ( (*p) && (*p == ' ') ) p++;

					if (*p != '-')
					{
						for (i=0; (i < 4) && (*p); i++)
						{
							while (*p && (isdigit(*p) == 0) ) p++;

							if (*p)
							{
								sscanf( p, "%d", &temp );
								gFlags.Myaddress  = ((gFlags.Myaddress << 8) & 0xFFFFFF00) | (temp & 0xFF);
							}

							while (*p && (*p != '.')) p++;
						}
					}

					break;
				}

/**** Check if user wants run quietly ****/

				case ('q'):
				{
					gFlags.Debug_level = 0;
					break;
				}

/***** Set the mode of program behaviour to memory dump ****/

				case ('s'):
				{
					gFlags.Mode |= MODE_SERVER_ONLY;
					break;
				}

/***** Force timer only connection monitoring ****/

				case ('t'):
				{
					if (gFlags.Idle == 0) gFlags.Idle = 15;

					gFlags.Mode |= MODE_TIMER_ONLY;
					break;
				}

/**** Indicate the version number to use ****/

				case ('v'):
				{
					gFlags.Show_version = TRUE;
					break;
				}

/***** Default action if none of the above can *****/

				default:
				{
					break;
				}
			}
		}

		while ( (*p) && (*p != ' ') ) p++;
	}

/**** Handle further builds on the PPPD command line ****/

	if ( (gFlags.Mode & MODE_DEVELOPMENT) &&
	    ((gFlags.Mode & MODE_TIMER_ONLY) == 0) )
	{
		strcpy( gFlags.Pppcmd, PPP_CMD1 );
	}

	if (gFlags.Idle > 0)
	{
		sprintf( idle_string, " idle %d ", gFlags.Idle );
		strcat( gFlags.Pppcmd, idle_string );
	}

/**** Show some of the command line settings ****/

	Dprintf( 2, "\tDebug_level    = %d\n", gFlags.Debug_level );
	Dprintf( 2, "\tppp command    = \"%s\"\n", gFlags.Pppcmd );
	Dprintf( 2, "\tDaddr          = \"%s\"\n", gFlags.Daddr );

	if (gFlags.Myaddress != -1)
	{
		Dprintf( 2, "\tMy IP address  = %d.%d.%d.%d\n",
		         gFlags.Myaddress >> 24,
				 (gFlags.Myaddress >> 16) & 0xFF,
				 (gFlags.Myaddress >> 8 ) & 0xFF,
				 (gFlags.Myaddress & 0xFF) );
	}

	if (gFlags.Mode & MODE_DEVELOPMENT)	Dprintf( 2, "\tDevelopment mode = ON\n" );
	else								Dprintf( 2, "\tDevelopment mode = OFF\n" );

	if (gFlags.Mode & MODE_MASTER_ON)	Dprintf( 2, "\tMaster mode = ON\n" );

	if (gFlags.Mode & MODE_MASTER_ON)	Dprintf( 2, "\tMaster mode = OFF\n" );

	if (gFlags.Mode & MODE_CONNECT)		Dprintf( 2, "\tConnect mode = ON\n" );

/**** Clean up and return answer to caller ****/

	return( arg_count );
}


/*********************************************************************
 This will walk the command argument list and search for specific
 whole word keywords. If any show up then a specific flag is set so
 that the main program can handle it.

 The arguments passed are the same as those given to main().

 Returns: Composite flag mask of fat words found.
**********************************************************************/

int	GetFatCommands( int argc, char ** argv )
{
	int	i;

	for (i=1; i < argc; i++)
	{
		if (argv[i] != NULL)
		{
			if (strcmp( argv[i], "development" ) == 0)	gFlags.Mode |= MODE_DEVELOPMENT;
			if (strcmp( argv[i], "master_on"  )  == 0)	gFlags.Mode |= MODE_MASTER_ON;
			if (strcmp( argv[i], "master_off" )  == 0)	gFlags.Mode |= MODE_MASTER_OFF;
			if (strcmp( argv[i], "server_only" ) == 0)	gFlags.Mode |= MODE_SERVER_ONLY;
			if (strcmp( argv[i], "help" )        == 0)	gFlags.Show_usage = TRUE;
			if (strcmp( argv[i], "version" )     == 0)	gFlags.Show_version = TRUE;

			if (strcmp( argv[i], "con" )         == 0)
			{
				gFlags.Mode |= MODE_CONNECT;

				if (argv[i+1] != NULL)
				{
					if (isxdigit(*argv[i+1]))
					{
						i++;
						strcpy( gFlags.Daddr, argv[i] );
					}
				}
			}
		}
	}

	return( gFlags.Mode );
}


/*********************************************************************
 This function shows the user how to use the supported soft switches
 of this software suite. You should call this function from the main
 function when help is needed by the user. The major functions of the
 main program must be display by that part.

 Also try to remember to update this module when making modifications
 to the function "GetOpt()" above as this function describes that 
 to the user.

 Returns: nothing.
**********************************************************************/

void	ShowSoftUsage()
{
	printf("\nUsage: %s -[d n][-i n][hqsv] <commands>\n\n", gFlags.Program );

	printf("\nCommand line Soft Switches:\n" );
 	printf("\t-d n                    - Set debug level to 'n'.\n" );
	printf("\t-D                      - Do not run as Daemon. Stay in forground.\n" );
	printf("\t-i n                    - Set PPPD idle timeout value.\n" );
	printf("\t-h                      - Show this help list.\n" );
	printf("\t-l                      - Turn off /etc/hosts file management.\n" );
	printf("\t-m <ipadrs>             - Force my IP adrs to <ipadrs>.\n" );
	printf("\t-q                      - turn off all reporting messages.\n" );
	printf("\t-s                      - Force program into server mode only.\n" );
	printf("\t-t                      - Turn off LCP pings and use idle timer monitoring.\n" );
	printf("\t-v                      - Show program revision level.\n" );
	printf( "\t%s  con             - Initiate a ppp connection to another device.\n", gFlags.Program );
	printf( "\t%s  development     - Go to auto seek and connect mode and watch link closely.\n", gFlags.Program );
	printf( "\t%s  master_on       - Make this device a IrLAP master.\n", gFlags.Program );
	printf( "\t%s  master_off      - Make this device a IrLAP slave.\n", gFlags.Program );
	printf( "\t%s  server_only     - Be a server only. Don't start connections.\n", gFlags.Program );
	printf( "\t%s  version         - Same as '-v' which shows revision number\n", gFlags.Program );
	printf( "\t%s  help            - This help message.\n", gFlags.Program );

 	printf("\nNote 1: When using these soft switches remember that character case is\n" );
 	printf("        important. eg: \"-a\" is not the same as \"-A\"\n" );
	printf("\nNote 2: [address] can be either hex which is preceeded by at 0x\n" );
	printf("                  or decimal.\n" );

 	return;
}


/*************************************************************************
 Use this function to read out a hexadecimal ASCII number from a string
 pointed to by the global control structure element gFlags.Cmd. A maximum
 of eight digits will be decoded to form an entire 32-bit number.

 The converted value will be returned from the function, while the global
 string pointer "gFlags.Cmd" will be updated to point to the next place
 after the end of the number.

 Returns: 32-bit integer.
**************************************************************************/

int	atox( char * string )
{
	 int	i;
	 int	n = 0;
	 int	t;
	char *	p;
	char *	pend;

/**** Find the end of the number ****/

	pend = string;

	for (p=string; 1; p++)
	{
		if ((*p == '\0') ||
		    (*p == ' ')  ||
			(*p == '\t') ||
			(*p == '\r') ||
			(*p == '\n') )
		{
			pend = p;
			p--;
			break;
		}
	}

/**** Convert the ASCII number to HEX ****/
	
	for (i=0; (i != 8) && (p >= string); i++, p--)
	{
		t = (int)toupper( *p );
		
		if ( (*p == '\t') ||
		     (*p == ' ')  ||
		     (t < '0')   ||
		     (t > 'F')   ||
		     ((t > '9') && (t < 'A')) )
		{
			break;
		}
		
		if ((t >= 'A') && (t <= 'F'))
		{
			n |= (((t - 'A') + 10) & 0x0F) << (i << 2);
		}
		else
		{
			if ((t >= '0') && (t <= '9'))
			{
				n |= (((t - '0') & 0x0f) << (i << 2));
			}
		}
	}

	return( n );	
}


/****************************  EOF  **********************************/




