/***************************************************************************
* Author:
*	-Kev	(mailto:kevin_bertram@hp.com)
*
*     Copyright (c) 2001 Hewlett Packard Pty. Ltd., All Rights Reserved.
*     
*     This program is free software; you can redistribute it and/or 
*     modify it under the terms of the GNU General Public License as 
*     published by the Free Software Foundation; either version 2 of 
*     the License, or (at your option) any later version.
* 
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*     GNU General Public License for more details.
* 
*     You should have received a copy of the GNU General Public License 
*     along with this program; if not, write to the Free Software 
*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
*     MA 02111-1307 USA
*
* FILE:
*   irnetd.c
*
****************************************************************************
* Purpose:
*
*   Automatic PPP connection using IrNET. This program listens to events
*	in /dev/irnet and creates a PPP connection if it detects a connection
*	request
*
*   The original concept was derived from:
*
*		Luiz Magalhaes - HPL '00 (luiz_magalhaes@hp.com)
*		Jean Tourrilhes - HPL '00 <jt@hpl.hp.com>
*
*	The node "/dev/irnet" is at major 10, minor 187.
*
*	There have been a few refinments made over the original version of
*	irnetd. They are to make IrDA IP networking much easier to implement
*	and are mostly controlled automatically by this daemon.
*
*	The list of features of irnetd are:
*
*	(1)	Automatic connect to remote device when discovered.
*	(2)	Two modes of operation applied to PPPD when started. One where
*		LCP pings are used to maintain connection monitoring, to the second
*		where IP traffic is used to define the life of PPPD.
*	(3)	Automatic connection, on request from a remote deivce, using PPPD.
*	(4)	IP address used for the PPPD (host side) is derived from the IrDA
*		MAC address (which is random) where the upper byte is fixed to 10.
*		This means that all IrDA devices will exist in the 10.x.x.x subnet.
*	(5)	/etc/hosts file managed to show each IrDA (PPPD) device attached to
*		this host. The entries look like "irdevice0, irdevice1 irdevice2, etc"
*		and are added on connection established, and removed on connection
*		exit.
*	(6)	Irnetd and be start as a server only so that it does not initiate
*		any new connections even when a remote device is discovered.
*	(7)	Irnetd can be instructed to establish a connection to a remote
*		device. Usfull when in "server_only" mode.
*
*	(8)	Irnetd maintains an internal database of all known PPP (IrDA)
*		connection to it. It used this to prevent multiple connectiond to
*		occur to other device in range.
*
* Typical Usage:
*
*	(1)	Start Irnetd as a proactive daemon:
*
*			irnetd development
*
*		Here irnetd will monitor and connect to any IrDA (inetd aware)
*		device that comes into range. It will default to using LCP pings
*		to monitor the connection.
*
*	(2)	Start irnetd as a proactive dameon using IP taffic monitoring:
*
*			irnetd development -t -i 10
*
*		Here irnetd will monitor and connect to any IrDA (irnet aware)
*		device that comes into range. It will using IP traffic (or the lack
*		of it) to determine of PPPD, and thus the connection, should remain.
*
*	(3)	Start irnetd as a server only. Don't be proactive:
*
*			irnetd server_only
*
*		Here irnetd will only respond to an irnet request to connect when
*		initiated by the remote device. LCP pings are used to monitor the
*		presence of the connect. This will not initiate any type of PPP
*		connection.
*
*	(4)	Make irnetd initiate a connection to a remote device:
*
*			irnetd con 0a123456
*
*		Here irnetd will connect to the remote IrDA device whose MAC address
*		is 0x0A123456. It will then setup a PPPD connection to the remote
*		device and monitor the connection via LCP pings.
*
*	In any of these cases the command line soft switch options "-t -i 10"
*	will swap connection monitoring from LCP pings over to IP traffic
*	monitoring. That is when IP traffic moves through it that will reset
*	the PPPD timeout counter and prevent disconnection for a further 
*	10 seconds (in this case).
*
****************************************************************************
* Revision History:
*
* Note: these revision numbers go up daily (almost) because the program is
*       being distributed to users as it is written. And thus I need a way
*       to distinguish the difference.
*
* Version 0.1.0			29-June-2001		-Kev
*	(1)	First writting.
*
* Version 0.1.1			2-July-2001			-Kev
*   (1)	Added connection to specific IrDA node address. So as new connection
*       requests arrive they are connected to rather than the same entry at 
*       the top of the list.
*
* Version 0.1.2			3-July-2001			-Kev
*   (1)	Used irnetd to monitor the beinging and end of an IrDA-PPP connection
*       so that the presence of a connection (Request from xxx) starts a PPP
*       connection while the absence of an IrDA device (Expired xxxx) removes
*       the PPP connection. This also means that LSAP caching must be turned
*       off in the IrDA protocol stack. It is a driver option.
*   (2)	Limited the number of simultaneous connections to a defined limit
*       that is defined as "MAX_CONNECTIONS"
*
* Version 0.1.3			4-July-2001			-Kev
*   (1)	Added Dprintf() function and converted all debug messages to display
*       them through this method.
*   (2) Removed "IDLE 10" option from pppd invocation to rely only on the
*       lcp ping efforts ppp does. This means that pppd will close a connection
*       if the link is down for (5 retries 2 secs apart) which equals 10 secs.
*
* Version 0.1.4         5-July-2001         -Kev
*   (1)	Added hosts file management. This means new connections are added to
*       it and old ones removed.
*
* Version 0.1.5			6-July-2001			-Kev
*   (1)	Command line options can now be used along side the "-" softswitches
*       so that special options can be enabled and disabled. As a result of
*       adding a command line parser most of the program is controlled by a
*       single "gFlags" control structure.
*
* Version 0.1.6         9-August-2001       -Kev
*   (1) Added an option (-l) to prevent modifications being made to the
*       /etc/hosts file. -l for no logging of host names. Normally the
*       program will write out to the hosts file a name and IP address
*       for each new device PPP connects to. When this option is given
*       the hosts file remains untouched and the device is not registered
*       anyway.
*   (2) The hosts file was being corrupted by the hosts file manager. It added
*       a new entry correctly but when the time come to remove it the new 
*       entry remained while all others were deleted. This is not good becuase
*       of the obvious. It also confused a lot of other applications relying
*       on /etc/hosts file being intact.
*   (3) Added the softswitch (-f) to turn on/off daemon mode.
*
* Version 0.1.7         10-August-2001      -Kev
*   (1) Names used in the /etc/hosts files have been converted to "irdevice".
*   (2) Added "linkname" option into pppd command line because it will register
*       into /var/run a name.pid file showing the existence of this instance.
*   (3) Changed (-f) softswitch to (-D).
*
* Version 0.1.8			5-October-2001		-Kev
*	(1)	Added feature to clean up the /etc/hosts file of "irdevice" entries
*		at program start. Sometimes, when this program is stopped artificially
*		and then restarted can leave an entry there. This effects the name
*		resolve while a device is not connected.
*
*	(2)	Added IP address assignment, as derived from the IrDA MAC address, to
*		force the upper byte to 10 so that even a truely random MAC address
*		can be made to fit this subnet (10.x.x.x).
*
*	(3)	Added "-m <adrs>" command line softswitch to force this program to
*		assume its IP address as given in the command line. So a line like
*		"-m 1.2.3.4" means this side will aloways have to IP address 1.2.3.4.
*
*	(4)	Broke up the all the code in this file to seperate smaller files. The
*		reason is obvious - it was getting too large.
*
*	(5)	Added function PruneAddress() to ensure that all IrDA MAC addresses
*		get used as a 10.x.x.x MAC address reguardless of if they care or not.
*
*	(6)	Put UNIX signal handling in so that it can catch termination signals
*		to force a cleanup.
*
*	(7)	Tied gFlags.Debug_level to Dprintf() so that the command line can
*		determine which level of debugging messages must be shown. Currently
*		the levels are:
*			0	- Nothing accept serious errors.
*			1	- General operations.
*			2	- General operations and actions.
*			3	- General operations and actions and data (Variables) contents.
*
* Version 1.0.0			8-October-2001		-Kev
*	(1)	First main release.
*****************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<signal.h>
#include	<unistd.h>
#include	<ctype.h>
#include	<stdarg.h>
#include	<sys/types.h>
#include	<sys/time.h>

/**** Read in locally defined includes for this module ****/

#include	"irnetd.h"
#include	"getopt.h"
#include	"files.h"

/**** Define local data types for use by this program ****/

typedef	struct
{
	  int	Debug;					/* Flag signals when debug messages are to be reported */
	  int	Count;					/* Count of devices currently connected */
	pid_t	Pid[ MAX_CONNECTIONS ];	/* Process ID of each PPP daemon */
	 char	Daddr[MAX_CONNECTIONS][32];
									/* IrDA address of device */
} PppType;


static	PppType		sPpp;		/* Controls PPP connection monitoring */


/*************************************************************************
                               MAIN
							   ====

 This is were it all begins. If there are no command line arguments then
 the program will be run as a daemon that watches the IrNET port and
 initiates a connection via PPPD.

 If there are command line arguments given then they are used to direct
 other activies that this program can do. So far the list of controls are:

 To get an idea of the modes and operation commands that can be fed into
 this program see the module "getopt.c". It has a lot of description about
 what can be done. Also type the command "irnetd help". This will also
 expalin a lot of the features that can be used.
**************************************************************************/

int main(int argc, char **argv)
{
	             int	i;
	             int	adrs = 0x0A000001;
	            FILE *	irnet;
	           pid_t	pid;
	            char	line[256];
	struct sigaction	signals_hup;
	struct sigaction	signals_quit;
	struct sigaction	signals_kill;

/**** Initiate the process control structure ****/

	sPpp.Count = 0;

	for (i=0; i < MAX_CONNECTIONS; i++)
	{
		sPpp.Pid[i] = 0;
		memset( sPpp.Daddr, 0, sizeof(sPpp.Daddr) );
	}

/**** Handle the commandline args off to the parser ****/

	GetOpt( argc, argv );

/**** Terminate the fixed part of the command ****/

	if (GetIpAddress( &adrs ) == FAILED)
	{
		printf( "\r%s: IrDA network interface is missing. Aborting!\n",
		        PROGRAM_NAME );
	}

/**** If the user wants program info then display it ****/

	if (gFlags.Show_version == TRUE)		/* Disable master mode */
	{
		printf( "Program revision level: %s\n", PROGRAM_REVISION );
		exit( 0 );
	}

	if (gFlags.Show_usage)
	{
		ShowSoftUsage();
		exit( 0 );
	}


/**** Check if there are command line Fat commands to handle ****/

	if (gFlags.Mode & MODE_CONNECT)			/* Initiate a ppp connection */
	{
		StartConnection( gFlags.Daddr );
		exit( 0 );
	}

	if (gFlags.Mode & MODE_MASTER_ON)		/* Enable master mode */
	{	
		MasterMode( ON );
		exit( 0 );
	}

	if (gFlags.Mode & MODE_MASTER_OFF)		/* Disable master mode */
	{
		MasterMode( OFF );
		exit( 0 );
	}

/**** Show the user my IP address ****/

	printf( "\r%s: My IP address = %d.%d.%d.%d\n",
	        PROGRAM_NAME,
			(adrs >> 24) & 0xFF,
			(adrs >> 16) & 0xFF,
			(adrs >> 8) & 0xFF,
			(adrs & 0xFF) );

/**** Open IrNET event channel ****/

	irnet = fopen( DEVICE_IRNET, "r" );

	if (irnet == NULL)
	{
		printf( "%s: Error opening \"%s\"\n", PROGRAM_NAME, DEVICE_IRNET );
		exit( 1 );
	}
	else
	{
/**** Ensure that this program runs as a daemon ****/

		if (gFlags.Mode & MODE_DAEMON)
		{
			pid = fork();

			if (pid) exit(0);
		}

/**** Setup the program signal handlers ****/

		signal( SIGINT, SigInt );

		sigfillset( &signals_hup.sa_mask );
		signals_hup.sa_handler = &SigHup;
		signals_hup.sa_flags   = SA_ONESHOT;
		sigaction( SIGHUP, &signals_hup, NULL );

		signals_quit.sa_handler = &SigQuit;
		signals_quit.sa_flags   = SA_ONESHOT;
		sigaction( SIGQUIT, &signals_quit, NULL );

		signals_kill.sa_handler = &SigQuit;
		signals_kill.sa_flags   = SA_ONESHOT;
		sigaction( SIGTERM, &signals_kill, NULL );


/**** Start the main loop of the program and watch that IrNET device ****/

		while(1)	/* Wait for one event */
		{
			fgets( line, 255, irnet );

			Dprintf( 2, "main(): IrNET message is: %s", line );

/**** If we have a request connect to it the connection will dye when there is no activity ****/

			if (strncmp( line, CMD_REQUEST, strlen(CMD_REQUEST) ) == 0)
			{
				ConnectOnRequest( line );
			}

/**** Check if the IrDA connection has terminated ****/

			if (strncmp( line, CMD_EXPIRED, strlen(CMD_EXPIRED) ) == 0)
			{
			}

/**** Check if another IrDA device has been found ****/

			if (strncmp( line, CMD_DISCOVERED, strlen(CMD_DISCOVERED) ) == 0)
			{
				if ((gFlags.Mode & MODE_SERVER_ONLY) == 0)
				{
					Connect( line );
				}
			}

/**** Check if another IrDA device has been found ****/

			if (strncmp( line, CMD_FOUND, strlen(CMD_FOUND) ) == 0)
			{
				if ((gFlags.Mode & MODE_SERVER_ONLY) == 0)
				{
					Connect( line );
				}
			}

/**** Check if a PPP connection has been established ****/

			if (strncmp( line, CMD_CONNECTION, strlen(CMD_CONNECTION) ) == 0)
			{
				ConnectionReport( line );
			}

/**** Check if a PPP connection has been disconnected ****/

			if (strncmp( line, CMD_DISCONNECTION, strlen(CMD_DISCONNECTION) ) == 0)
			{
				DisconnectReport( line );
			}

/**** Check if a PPP connection has been blocked or interfered with ****/

			if (strncmp( line, CMD_BLOCKED, strlen(CMD_BLOCKED) ) == 0)
			{
			}
		}
	}
}


/***********************************************************************
 When this function is called it can be assumed that the user, or kernel
 has issued a kill command (^c). It will be the job of this function to
 clean up the system then exit.

 Returns: Back to the OS.
************************************************************************/

void	SigInt( int sig )
{
	signal( SIGINT, SIG_DFL );
	Dprintf( 2, "Got INT signal. Cleaning up...\n" );

	CleanHostFile();
	exit( 0 );
}

/***********************************************************************
 This will be called if the system has issued a SIGHUP is pushed into
 the program.
************************************************************************/

void	SigHup( int sig )
{
	signal( SIGINT, SIG_DFL );
	Dprintf( 2, "Got HUP signal. Cleaning up...\n" );

	CleanHostFile();
	exit( 0 );
}

/***********************************************************************
 This will be called when the program receives a (^z) sleep-stop command.
************************************************************************/

void	SigQuit( int sig )
{
	signal( SIGINT, SIG_DFL );
	Dprintf( 2, "Got TERM signal. Cleaning up...\n" );

	CleanHostFile();
	exit( 0 );
}



/*************************************************************************
 This function will connect to an IrDA device that has just been discovered
 only if this machine is a "Master". The master initiates an automatic
 connection to prevent two simultaneous connections to be initated - one
 at each end.

 Returns: TRUE		- Connection established
		  FAILED	- Could not connect.
**************************************************************************/

int	Connect( char * command )
{
	         int	return_code = FAILED;
	         int	i;
	unsigned int	adrs = 0;
	         int	slot;
	        char *	ptr;
	        char	node[80];
	        char	cmd[256];
	        char	ipadrs[40];

	Dprintf( 2, "Connect(): IrNET said:%s", command );

/**** Check that the maximum connected devices to this machine ****/

	if (sPpp.Count >= MAX_CONNECTIONS)
	{
		Dprintf( 1, "Connect(): Refusing connection because there are already too many.\n" );
		return( FAILED );
	}

/**** Get nodes name ****/

	if (IsMaster() == TRUE)
	{
		ptr = command;
		memset( node, 0, sizeof(node) );

		while ((*ptr) && (*ptr != ' ')) ptr++;
		while ((*ptr) && (*ptr == ' ')) ptr++;

		for (i=0; (i < sizeof(node)) && (*ptr) && (*ptr != ' '); i++)
		{
			node[i] = *ptr++;
			node[i+1] = '\0';
		}

/**** Build command string based on our current IP->MAC address ****/

		if (GetIpAddress( &adrs ) == FAILED)
		{
			printf( "%s: IrDA network interface is missing - Aborting connect attempt!\n",
					PROGRAM_NAME );

			return( FAILED );
		}

		sprintf( ipadrs, "%d.%d.%d.%d",
				 (adrs >> 24) & 0xFF,
				 (adrs >> 16) & 0xFF,
				 (adrs >> 8) & 0xFF,
				 (adrs & 0xFF) );

		sprintf( cmd, "%s linkname %s %s: connect \"echo addr %s\" &",
				 gFlags.Pppcmd, 
				 node,
				 ipadrs,
				 node );

		slot = AddConnectEntry( node );

		if (slot != -1)
		{
			Dprintf( 2, "Connect(): cmd=\"%s\"\n", cmd );

			system( cmd );

			sprintf( cmd, "Connect from %s (myip=%d.%d.%d.%d)",
					 node,
					 (adrs >> 24) & 0xFF,
					 (adrs >> 16) & 0xFF,
					 (adrs >> 8)  & 0xFF,
					 (adrs & 0xFF) );

			ConnectionReport( cmd );
		}
		else
		{
			Dprintf( 1, "Connect(): Already connected to \"%s\"\n", node );
		}
	}
	else
	{
		Dprintf( 1, "Connect() I'm not a MASTER!\n" );
	}

	return( return_code );
}


/*************************************************************************
 This function will attempt to disconnect a PPP connection to a remote 
 device based on its IrDA address. The address is stored at connect time
 and also the PID of the PPP daemon that is connected to the device.

 The PPP daemon will be given a signal to hungup the connection and there
 fore release the slot.

 Returns: TRUE		- Connection removed
		  FAILED	- Could not find and remove the PPP connection.
**************************************************************************/

int	DisconnectReport( char * command )
{
	 int	return_code = FAILED;
	 int	i;
	char *	ptr;
	char	node[80];

	Dprintf( 2, "DisconnectReport(): Disconnect(): command=%s", command );

/**** Get nodes name ****/

	ptr = command;
	memset( node, 0, sizeof(node) );

	while ((*ptr) && (*ptr != ' ')) ptr++;
	while ((*ptr) && (*ptr == ' ')) ptr++;

	if (strncmp( ptr, "from", 4 ) == 0)
	{
		while ((*ptr) && (*ptr != ' ')) ptr++;
		while ((*ptr) && (*ptr == ' ')) ptr++;

		for (i=0; (i < sizeof(node)) && (*ptr) && (*ptr != ' '); i++)
		{
			node[i] = *ptr++;
			node[i+1] = '\0';
		}

		RemoveFromHosts( command );

		return_code = RemoveConnectEntry( node );

		if (return_code == -1)
			Dprintf( 2, "DisconnectReport(): Node \"%s\" does not exist to delete!\n", node );
		else
			Dprintf( 2, "DisconnectReport(): Removing \"%s\" from entry[%d]\n", node, return_code );
	}

	return( return_code );
}


/*************************************************************************
 This function is called when a connection is reported by IrNET. The signal
 represents a successfull PPP end-to-end connection from another device.

 This will add the connection request into the connect table if it is not
 there.

 Returns: TRUE		- Connection removed
		  FAILED	- Could not find and remove the PPP connection.
**************************************************************************/

int	ConnectionReport( char * command )
{
	 int	return_code = FAILED;
	 int	i;
	char *	ptr;
	char	node[80];

	Dprintf( 2, "ConnectionReport(): command=%s", command );

/**** Get nodes name ****/

	ptr = command;
	memset( node, 0, sizeof(node) );

	while ((*ptr) && (*ptr != ' ')) ptr++;
	while ((*ptr) && (*ptr == ' ')) ptr++;

	if (strncmp( ptr, "from", 4 ) == 0)
	{
		while ((*ptr) && (*ptr != ' ')) ptr++;
		while ((*ptr) && (*ptr == ' ')) ptr++;

		for (i=0; (i < sizeof(node)) && (*ptr) && (*ptr != ' '); i++)
		{
			node[i] = *ptr++;
			node[i+1] = '\0';
		}

/**** If we get here then there is no matching entry so we should add one ****/

		return_code = AddConnectEntry( node );

		if (return_code >= 0) AddToHosts( command );

		Dprintf( 2, "ConnectionReport(): Node \"%s\" entered as item[%d]\n", node, return_code );
	}

	return( return_code );
}


/*************************************************************************
 This command initiates a connection with another device. It will build
 a pppd invocation command and include its IP address (static) as well. It
 also assumes that the other device knows its own IP address and has 
 assigned it there.

 If all works will then there will be a ppp connection between us and the
 other device.

 Returns:	TRUE	- OK
			FAILED	- Could not start connection.
**************************************************************************/

int	StartConnection( char * node )
{
	         int	adrs = 0x0A000000;
	unsigned int	timer;
	        char	cmd[256];
	        char	ipadrs[40];

/**** Check that the maximum connected devices to this machine ****/

	if (sPpp.Count >= MAX_CONNECTIONS)
	{
		Dprintf( 1, "StartConnection(): Refusing connection because there are already too many.\n" );
		return( FAILED );
	}

/**** Build command string based on our current IP->MAC address ****/

	if (GetIpAddress( &adrs ) == FAILED)
	{
		printf( "%s: IrDA network interface is missing - Aborting connect attempt!\n",
				PROGRAM_NAME );

		return( FAILED );
	}

	sprintf( ipadrs, "%d.%d.%d.%d",
			 (adrs >> 24) & 0xFF,
			 (adrs >> 16) & 0xFF,
			 (adrs >> 8) & 0xFF,
			 (adrs & 0xFF) );

	sprintf( cmd, "%s linkname %s %s: connect \"echo addr %s\" &",
		     gFlags.Pppcmd,
			 node,
			 ipadrs,
			 node );

/**** Check if there are IrLAP connections to talk to ****/

	if (IsIrdaDevices() == 0)
	{
		if (IsMaster() == TRUE)
		{
			return( FAILED );
		}

		MasterMode( ON );
		timer = ProcTimePlus( ProcTime(), ProcClockSecond() * 5 );

		while (1)
		{
			ProcWait( ProcClockSecond() /*/ 10*/ );

			if (IsIrdaDevices())
			{
				break;
			}

			if (ProcTimeAfter( ProcTime(), timer )) return( FAILED );
		}
	}

/**** Run pppd to handle connection ****/

	Dprintf( 2, "Con command is \"%s\"\n", cmd );

	if (system(cmd) < 0)
	{
		perror("irnetd: ");
	}
	else
	{
		sprintf( cmd, "Connect from %s (myip=%d.%d.%d.%d)",
		         node,
				 (adrs >> 24) & 0xFF,
				 (adrs >> 16) & 0xFF,
				 (adrs >> 8)  & 0xFF,
				 (adrs & 0xFF) );

		ConnectionReport( cmd );
	}

	return( TRUE );
}


/*************************************************************************
 This command should be called to answer a connection request. It will
 build a pppd invocation line and include this devices IP address into
 it. It is assumed that the other device knows its IP address so we don't
 force the issue here.

 Returns:	TRUE	- OK
			FAILED	- Could not start connection.
**************************************************************************/

int	ConnectOnRequest( char * devcmd )
{
	 int	i;
	 int	adrs = 0x0A000000;
	char *	ptr;
	char	node[80];
	char	cmd[256];
	char	ipadrs[40];

	Dprintf( 2, "ConnectOnRequest(): ConnectOnRequest(): command=%s", devcmd );

/**** Check that the maximum connected devices to this machine ****/

	if (sPpp.Count >= MAX_CONNECTIONS)
	{
		Dprintf( 1, "ConnectOnRequest(): Refusing connection because there are already too many.\n" );
		return( FAILED );
	}

/**** Get nodes name ****/

	ptr = devcmd;
	memset( node, 0, sizeof(node) );

	while ((*ptr) && (*ptr != ' ')) ptr++;
	while ((*ptr) && (*ptr == ' ')) ptr++;

	if (strncmp( ptr, "from", 4 ) == 0)
	{
		while ((*ptr) && (*ptr != ' ')) ptr++;
		while ((*ptr) && (*ptr == ' ')) ptr++;

		for (i=0; (i < sizeof(node)) && (*ptr) && (*ptr != ' '); i++)
		{
			node[i] = *ptr++;
			node[i+1] = '\0';
		}
	}

/**** Build command string based on our current IP->MAC address ****/

	if (GetIpAddress( &adrs ) == FAILED)
	{
		Dprintf( 1, "ConnectOnRequest(): IrDA network interface is missing - Aborting connect attempt!\n" );

		return( FAILED );
	}

	sprintf( ipadrs, "%d.%d.%d.%d",
			 (adrs >> 24) & 0xFF,
			 (adrs >> 16) & 0xFF,
			 (adrs >> 8) & 0xFF,
			 (adrs & 0xFF) );

	sprintf( cmd, "%s linkname %s %s: connect \"echo addr %s\" &",
	         gFlags.Pppcmd,
			 node,
			 ipadrs,
			 node );

/**** Run pppd to handle connection ****/

	Dprintf( 2, "ConnectOnRequest(): Command = %s\n", cmd );

	if (system(cmd) < 0)
	{
		perror("irnetd: ");
	}
	else
	{
		ConnectionReport( devcmd );
	}

	return( TRUE );
}

/*****************************************************************************
 Returns current timer from power up to now in counts of 10 milliseconds. This
 can be used to provide a polled timer type calling mechanism which does not
 require the caller to wait, or sleep, on a timeout interval.

 Returns: timer count in 10 milliseconds.
******************************************************************************/

long ProcTime( void )
{
	struct timeval	tv;
	struct timezone	tz;

	gettimeofday( &tv, &tz );

	return( (tv.tv_sec * 100) + (tv.tv_usec / 10000) );
}


/**************************************************************************
 This function returns the number of ticks per second the system timer is
 calibrated to.
 
 Returns: Count of ticks per second.
***************************************************************************/

long	ProcClockSecond( void )
{
	return( 100 );
}

/**************************************************************************
 Is used to compare to system tick time values and returns and answer based
 on that comparison.
 
 Returns:
 	TRUE	- "timer2" is after "time1"
 	FALSE	- Not yet.
***************************************************************************/

int	ProcTimeAfter( long time1, long time2 )
{
	time2 -= time1;
	
	if (time2 & 0x8000000)	return( TRUE );
	else					return( FALSE );
}

/**************************************************************************
 Adds two system tick time values together and returns the result. On an
 ARM system this function is redundant, however, on a Transputer based 
 system this function matters. It is needed because adding signed numbers
 together causes an overflow condition which is an error for the Transputer
 and will halt the system.
 
 Returns: "time1" + "time2"
***************************************************************************/

long	ProcTimePlus( long time1, long time2 )
{
	return( time1 + time2 );
}


/**************************************************************************
 This function waits for a defined period of time before exiting. It is
 syncronised to the ProcTime() function and will used its time value based
 on that timebase.

 Returns: Nothing
***************************************************************************/

void	ProcWait( long timer )
{
	timer = ProcTimePlus( ProcTime(), timer );

	while (ProcTimeAfter( timer, ProcTime() )) ;
}


/***************************************************************************
 Searches throught the active connection list for a matching node address.
 The node address is the IrDA MAC address used to identify IrDA devices.

 Returns:	>= 0	ID of matching entry.
			-1		No matching entry found.
****************************************************************************/

int	FindConnectEntry( char * node )
{
	int	i;
	int	slot = -1;

	Dprintf( 3, "FindConnectEntry(): Finding node adrs \"%s\"\n", node );

	for (i=0; i < MAX_CONNECTIONS; i++)
	{
		Dprintf( 3, "FindConnectEntry(): [%d] \"%s\"\n", i, sPpp.Daddr[i] );

		if (strncmp( node, sPpp.Daddr[i], sizeof(sPpp.Daddr[i]) ) == 0)
		{
			slot = i;
			break;
		}
	}

	return( slot );
}


/***************************************************************************
 This adds a new node entry into the active connection list if there is not
 one there already.

 Returns:	>= 0	ID of matching entry just added.
			-1		Entry already exists.
****************************************************************************/

int	AddConnectEntry( char * node )
{
	int	i;
	int	slot = -1;

	if (strlen( node ) > 0)
	{
		slot = FindConnectEntry( node );

		if (slot == -1)
		{
			for (i=0; i < MAX_CONNECTIONS; i++)
			{
				if (strlen(sPpp.Daddr[i]) == 0)
				{
					if ((sPpp.Count > 0) && (sPpp.Pid == 0)) sPpp.Count++;

					slot = i;
					sPpp.Pid[i] = 1;
					memset( sPpp.Daddr[i], 0, sizeof(sPpp.Daddr[i]) );
					strncpy( sPpp.Daddr[i], node, sizeof(sPpp.Daddr[i]) );
					break;
				}
			}
		}
	}

	return( slot );
}


/***************************************************************************
 This searches out an entry in the active connection list and removes it.

 Returns:	>= 0	ID of matching entry just deleted.
			-1		No matching entry found.
****************************************************************************/

int	RemoveConnectEntry( char * node )
{
	int	slot = -1;

	if (strlen( node ) > 0)
	{
		slot = FindConnectEntry( node );

		Dprintf( 3, "RemoveConnectEntry(): Deleting entry[%d] = \"%s\"\n", slot, sPpp.Daddr[slot] );

		if (slot >= 0)
		{
			if (sPpp.Count > 0) sPpp.Count--;

			sPpp.Pid[slot] = 0;
			memset( sPpp.Daddr[slot], 0, sizeof(sPpp.Daddr[slot]) );
		}
	}

	return( slot );
}


/******************************************************************************
 This function does some the basics that you would expect from a "printf()"
 like function. Its purpose is to cruedly assemble the given control string
 along with its argument list then send it to the console port, RS232 serial
 port.

 The settings of the serial port are defined and setup before this function is
 called.

 No numeral formatting controls exist as yet. Maybe in the future.

 returns:	Pointer to the formatted string.
*******************************************************************************/

#define	MAX_STRING_SIZE	1024


char *	Dprintf( int level, char * control_string, ... )
{
	        int		i;
	    va_list		vaptr;
	static char		temp_string[MAX_STRING_SIZE];

	temp_string[0] = '\0';

	if (level <= gFlags.Debug_level)
	{
/**** Get the start of the variable argument list ****/

		va_start( vaptr, control_string );

/**** Process control string and expand it with variable argument list ****/

		i = vsprintf( temp_string, control_string, vaptr );

/**** Clean up and return to caller ****/

		va_end( vaptr );
		printf( "\rDEBUG: %s", temp_string );
	}

	return( temp_string );
}



/*********************************  EOF  *********************************/

